@@ -1,73 +1,66 @@
 #
-# filelock.pl - functions for filelock on local or nfs server
+# filelock.pl - filelock routines for local or nfs server
 #
 # 2001/04/25 tung@turtle.ee.ncku.edu.tw
 #
+package openwebmail::filelock;
+
 use strict;
 use Fcntl qw(:DEFAULT :flock);
-use FileHandle;
-
-sub filelock {
-   my $ret;
-   if ( $::config{'use_dotlockfile'} ) {
-      $ret=filelock_dotlockfile(@_);
-   } else {
-      $ret=filelock_flock(@_);
-   }
-   return($ret);
-}
 
 # this routine provides flock with filename
 # it opens the file to get the handle if need,
 # than do lock operation on the related filehandle
-my %opentable;
-sub filelock_flock {
+use vars qw(%opentable);
+%opentable=();
+
+sub flock_lock {
    my ($filename, $lockflag)=@_;
    my ($dev, $inode, $fh);
 
+   ($filename =~ /^(.+)$/) && ($filename = $1);	# untaint...
+
    if ( (! -e $filename) && $lockflag ne LOCK_UN) {
-      ($filename =~ /^(.+)$/) && ($filename = $1);   
       sysopen(F, $filename, O_RDWR|O_CREAT, 0600); # create file for lock
       close(F);
-   } 
+   }
 
    ($dev, $inode)=(stat($filename))[0,1];
    if ($dev eq '' || $inode eq '') {
-      return(0);
+      return 0;
    }
 
-   if (defined($opentable{"$dev-$inode"}) ) {	
+   if (defined($opentable{"$dev-$inode"}) ) {
       $fh=$opentable{"$dev-$inode"};
    } else { # handle not found, open it!
-      $fh=FileHandle->new();
-      if (sysopen($fh, $filename, O_RDWR)) {
+      $fh=do { local *FH };
+      if (sysopen($fh, $filename, O_RDWR) ||	# try RDWR open first
+          sysopen($fh, $filename, O_RDONLY) ) {	# then RDONLY for readonly file
          $opentable{"$dev-$inode"}=$fh;
       } else {
-         return(0);
+         return 0;
       }
    }
 
-   # Since nonblocking lock may return errors 
-   # even the target is locked by others for just a few seconds,
-   # we turn nonblocking lock into a blocking lock with timeout limit=60sec
-   # thus the lock will have more chance to success.
-
-   if ( $lockflag & LOCK_NB ) {	# nonblocking lock
-      my $retval;
-      eval {
-         local $SIG{ALRM} = sub { die "alarm\n" }; # NB: \n required
-         alarm 60;
-         $retval=flock($fh, $lockflag & (~LOCK_NB) );	
-         alarm 0;
-      };
-      if ($@) {	# eval error, it means timeout
-         $retval=0;
+   # turn nonblocking lock to  30 secs timeouted lock
+   # so owm gets higher chance to success in case other ap locks same file for only few secs
+   # turn blocking    lock to 120 secs timeouted lock
+   # so openwebmaill won't hang because of file locking
+   my $retval;
+   eval {
+      local $SIG{ALRM} = sub { die "alarm\n" }; # NB: \n required
+      if ( $lockflag & LOCK_NB ) {	# nonblocking lock
+         alarm 30;
+      } else {
+         alarm 120;
       }
-      return($retval);
-
-   } else {			# blocking lock				
-      return(flock($fh, $lockflag));
+      $retval=flock($fh, $lockflag & (~LOCK_NB) );
+      alarm 0;
+   };
+   if ($@) {	# eval error, it means timeout
+      $retval=0;
    }
+   return($retval);
 }
 
 
@@ -75,27 +68,35 @@ sub filelock_flock {
 # it is only recommended if the files are located on remote nfs server
 # and the lockd on your nfs server or client has problems
 # since it is slower than flock
-sub filelock_dotlockfile {
+#
+# This routine stores the shared lock counter in filename.lock and
+# uses filename.lock.lock to guarentee unique access to filename.lock.
+# If the filename is locates in a readonly directory, since the
+# filename.lock and filename.lock.lock could not be created, this routine will
+# grant all shared lock and deny all exclusive lock.
+sub dotfile_lock {
    my ($filename, $lockflag)=@_;
    my ($mode, $count);
 
    return 1 unless ($lockflag & (LOCK_SH|LOCK_EX|LOCK_UN));
 
    my $endtime;
-   if ($lockflag & LOCK_NB) {	# turn nonblock lock to 60sec blocking lock
-      $endtime=time()+60;
-   } else {
-      $endtime=time()+86400;
+   if ($lockflag & LOCK_NB) {	# turn nonblock lock to 30sec blocking lock
+      $endtime=time()+30;
+   } else {			# turn blocking lock to 120sec blocking lock
+      $endtime=time()+120;
    }
 
    my $oldumask=umask(0111);
 
    # resolve symbolic link
+   my $ldepth=0;
    while (-l "$filename") {
+      $ldepth++; return (0) if ($ldepth>32);		# link to deep
       $filename=readlink($filename);
    }
    ($filename =~ /^(.+)$/) && ($filename = $1);		# untaint ...
-   
+
    while (time() <= $endtime) {
       my $status=0;
 
@@ -104,9 +105,16 @@ sub filelock_dotlockfile {
          unlink("$filename.lock") if (time()-$t > 300);
       }
 
-      if (_lock("$filename.lock")==0) {
+      my $locklock=_lock("$filename.lock");
+      if ($locklock==0) {
          sleep 1;
          next;
+      } elsif ($locklock==-1) {	# rdonly dir, no further processing
+         if ($lockflag & LOCK_EX) {
+            return 0;
+         } else {
+            return 1;
+         }
       }
 
       if ( $lockflag & LOCK_UN ) {
@@ -169,17 +177,17 @@ sub filelock_dotlockfile {
       if ($status==1) {
          _unlock("$filename.lock");
          umask($oldumask);
-         return(1);
+         return 1;
       } else {
          _unlock("$filename.lock");
          sleep 1;
          next;
       }
-   }   
+   }
 
    _unlock("$filename.lock");
    umask($oldumask);
-   return(0);
+   return 0;
 }
 
 
@@ -195,19 +203,28 @@ sub _lock {
    }
    if ( sysopen(LL, "$filename.lock", O_RDWR|O_CREAT|O_EXCL) ) {
       close(LL);
-      return(1)
+      return 1
    } else {
-      return(0);
+      if ($!=~/permission/i) {	# .lock file in readonly dir?
+         return -1;
+      } else {			# .lock file already exist
+         return 0;
+      }
    }
 }
+
 sub _unlock {
    my ($filename)=$_[0];
    ($filename =~ /^(.+)$/) && ($filename = $1);		# untaint ...
 
    if ( unlink("$filename.lock") ) {
-      return(1);
+      return 1;
    } else {
-      return(0);
+      if ($!=~/permission/i) {	# .lock file in readonly dir?
+         return -1;
+      } else {
+         return 0;
+      }
    }
 }
 
