@@ -1,58 +1,57 @@
 #!/usr/bin/perl -T
-#############################################################################
-# Open WebMail - Provides a web interface to user mailboxes                 #
-#                                                                           #
-# Copyright (C) 2001-2002                                                   #
-# Chung-Kie Tung, Nai-Jung Kuo, Chao-Chiu Wang, Emir Litric                 #
-# Copyright (C) 2000                                                        #
-# Ernie Miller  (original GPL project: Neomail)                             #
-#                                                                           #
-# This program is distributed under GNU General Public License              #
-#############################################################################
+#
+# openwebmail-folder.pl - mail folder management program
+#
 
-local $SCRIPT_DIR="";
-if ( $ENV{'SCRIPT_FILENAME'} =~ m!^(.*?)/[\w\d\-]+\.pl! || $0 =~ m!^(.*?)/[\w\d\-]+\.pl! ) { $SCRIPT_DIR=$1; }
-if (!$SCRIPT_DIR) { print "Content-type: text/html\n\n\$SCRIPT_DIR not set in CGI script!\n"; exit 0; }
-
-use strict;
-no strict 'vars';
-use Fcntl qw(:DEFAULT :flock);
-use CGI qw(:standard);
-use CGI::Carp qw(fatalsToBrowser);
-CGI::nph();   # Treat script as a non-parsed-header script
+use vars qw($SCRIPT_DIR);
+if ( $0 =~ m!^(.*?)/[\w\d\-\.]+\.pl! ) { $SCRIPT_DIR=$1; }
+if (!$SCRIPT_DIR && open(F, '/var/www/conf/openwebmail/openwebmail_path.conf')) {
+   $_=<F>; close(F); if ( $_=~/^([^\s]*)/) { $SCRIPT_DIR=$1; }
+}
+if (!$SCRIPT_DIR) { print "Content-type: text/html\n\nSCRIPT_DIR not set in /etc/openwebmail_path.conf !\n"; exit 0; }
+push (@INC, $SCRIPT_DIR);
 
 $ENV{PATH} = ""; # no PATH should be needed
+$ENV{ENV} = "";      # no startup script for sh
 $ENV{BASH_ENV} = ""; # no startup script for bash
-umask(0007); # make sure the openwebmail group can write
+umask(0002); # make sure the openwebmail group can write
 
-push (@INC, $SCRIPT_DIR, ".");
-require "openwebmail-shared.pl";
+use strict;
+use Fcntl qw(:DEFAULT :flock);
+use CGI qw(-private_tempfiles :standard);
+use CGI::Carp qw(fatalsToBrowser carpout);
+
+require "ow-shared.pl";
 require "filelock.pl";
 require "mime.pl";
 require "maildb.pl";
 
-local (%config, %config_raw);
-local $thissession;
-local ($loginname, $domain, $user, $userrealname, $uuid, $ugid, $homedir);
-local (%prefs, %style);
-local ($lang_charset, %lang_folders, %lang_sortlabels, %lang_text, %lang_err);
-local ($folderdir, @validfolders, $folderusage);
-local ($folder, $printfolder, $escapedfolder);
+use vars qw(%config %config_raw);
+use vars qw($thissession);
+use vars qw($domain $user $userrealname $uuid $ugid $homedir);
+use vars qw(%prefs %style %icontext);
+use vars qw($folderdir @validfolders $folderusage);
+use vars qw($folder $printfolder $escapedfolder);
 
 openwebmail_init();
-verifysession();
 
-local $firstmessage;
-local $sort;
+use vars qw($sort $page);
 
-$firstmessage = param("firstmessage") || 1;
-$sort = param("sort") || $prefs{"sort"} || 'date';
+$page = param("page") || 1;
+$sort = param("sort") || $prefs{'sort'} || 'date';
+
+# extern vars
+use vars qw(%lang_folders %lang_text %lang_err);	# defined in lang/xy
+use vars qw($_OFFSET $_STATUS);				# defined in maildb.pl
 
 ########################## MAIN ##############################
-
 my $action = param("action");
 if ($action eq "editfolders") {
    editfolders();
+} elsif ($action eq "refreshfolders") {
+   refreshfolders();
+} elsif ($action eq "markreadfolder") {
+   markreadfolder();
 } elsif ($action eq "chkindexfolder") {
    reindexfolder(0);
 } elsif ($action eq "reindexfolder") {
@@ -68,17 +67,22 @@ if ($action eq "editfolders") {
 } else {
    openwebmailerror("Action $lang_err{'has_illegal_chars'}");
 }
+
+# back to root if possible, required for setuid under persistent perl
+$<=0; $>=0;
 ###################### END MAIN ##############################
 
 #################### EDITFOLDERS ###########################
 sub editfolders {
    my (@defaultfolders, @userfolders);
-   local ($total_newmessages, $total_allmessages, $total_foldersize)=(0,0,0);
+   my $total_newmessages=0;
+   my $total_allmessages=0;
+   my $total_foldersize=0;
 
-   push(@defaultfolders, 'INBOX', 
-                         'saved-messages', 
-                         'sent-mail', 
-                         'saved-drafts', 
+   push(@defaultfolders, 'INBOX',
+                         'saved-messages',
+                         'sent-mail',
+                         'saved-drafts',
                          'mail-trash');
 
    foreach (@validfolders) {
@@ -91,23 +95,14 @@ sub editfolders {
       }
    }
 
-   my $html = '';
-   my $temphtml;
-
-   open (EDITFOLDERSTEMPLATE, "$config{'ow_etcdir'}/templates/$prefs{'language'}/editfolders.template") or
-      openwebmailerror("$lang_err{'couldnt_open'} $config{'ow_etcdir'}/templates/$prefs{'language'}/editfolders.template!");
-   while (<EDITFOLDERSTEMPLATE>) {
-      $html .= $_;
-   }
-   close (EDITFOLDERSTEMPLATE);
-
+   my ($html, $temphtml);
+   $html = readtemplate("editfolders.template");
    $html = applystyle($html);
 
    $html =~ s/\@\@\@FOLDERNAME_MAXLEN\@\@\@/$config{'foldername_maxlen'}/g;
 
-   printheader();
-
-   $temphtml = qq|<a href="$config{'ow_cgiurl'}/openwebmail-main.pl?action=displayheaders&amp;sessionid=$thissession&amp;sort=$sort&amp;firstmessage=$firstmessage&amp;folder=$escapedfolder" title="$lang_text{'backto'} $printfolder"><IMG SRC="$config{'ow_htmlurl'}/images/iconsets/$prefs{'iconset'}/backtofolder.gif" border="0" ALT="$lang_text{'backto'} $printfolder"></a>|;
+   $temphtml = iconlink("backtofolder.gif", "$lang_text{'backto'} $printfolder", qq|accesskey="B" href="$config{'ow_cgiurl'}/openwebmail-main.pl?action=listmessages&amp;sessionid=$thissession&amp;sort=$sort&amp;page=$page&amp;folder=$escapedfolder"|). qq|&nbsp; \n|;
+   $temphtml .= iconlink("refresh.gif", $lang_text{'refresh'}, qq|accesskey="R" href="$config{'ow_cgiurl'}/openwebmail-folder.pl?action=refreshfolders&amp;sessionid=$thissession&amp;sort=$sort&amp;folder=$escapedfolder&amp;page=$page"|). qq| \n|;
 
    $html =~ s/\@\@\@MENUBARLINKS\@\@\@/$temphtml/g;
 
@@ -121,24 +116,24 @@ sub editfolders {
                hidden(-name=>'sort',
                       -default=>$sort,
                       -override=>'1') .
-               hidden(-name=>'firstmessage',
-                      -default=>$firstmessage,
+               hidden(-name=>'page',
+                      -default=>$page,
                       -override=>'1') .
                hidden(-name=>'folder',
                       -default=>$folder,
                       -override=>'1');
-
    $html =~ s/\@\@\@STARTFOLDERFORM\@\@\@/$temphtml/;
 
    $temphtml = textfield(-name=>'foldername',
                          -default=>'',
-                         -size=> $config{'foldername_maxlen'},
+                         -size=> 24,
                          -maxlength=>$config{'foldername_maxlen'},
                          -override=>'1');
-
+#                         -accesskey=>'I',
    $html =~ s/\@\@\@FOLDERNAMEFIELD\@\@\@/$temphtml/;
 
    $temphtml = submit(-name=>"$lang_text{'add'}",
+                      -accesskey=>'A',
                       -class=>"medtext");
    $html =~ s/\@\@\@ADDBUTTON\@\@\@/$temphtml/;
 
@@ -147,29 +142,31 @@ sub editfolders {
 
    my $bgcolor = $style{"tablerow_dark"};
    my $currfolder;
-   my $i=0;
+   my $form_i=0;
    $temphtml='';
    foreach $currfolder (@userfolders) {
-      $temphtml .= _folderline($currfolder, $i, $bgcolor);
+      $temphtml .= _folderline($currfolder, $form_i, $bgcolor, 
+                               \$total_newmessages, \$total_allmessages, \$total_foldersize);
       if ($bgcolor eq $style{"tablerow_dark"}) {
          $bgcolor = $style{"tablerow_light"};
       } else {
          $bgcolor = $style{"tablerow_dark"};
       }
-      $i++;
+      $form_i++;
    }
    $html =~ s/\@\@\@FOLDERS\@\@\@/$temphtml/;
 
    $bgcolor = $style{"tablerow_dark"};
    $temphtml='';
    foreach $currfolder (@defaultfolders) {
-      $temphtml .= _folderline($currfolder, $i, $bgcolor);
+      $temphtml .= _folderline($currfolder, $form_i, $bgcolor,
+                               \$total_newmessages, \$total_allmessages, \$total_foldersize);
       if ($bgcolor eq $style{"tablerow_dark"}) {
          $bgcolor = $style{"tablerow_light"};
       } else {
          $bgcolor = $style{"tablerow_dark"};
       }
-      $i++;
+      $form_i++;
    }
    $html =~ s/\@\@\@DEFAULTFOLDERS\@\@\@/$temphtml/;
 
@@ -184,50 +181,49 @@ sub editfolders {
       $usagestr="&nbsp;";
    }
 
-   if ($total_foldersize > 1048575){
-      $total_foldersize = int(($total_foldersize/1048576)+0.5) . "MB";
-   } elsif ($total_foldersize > 1023) {
-      $total_foldersize =  int(($total_foldersize/1024)+0.5) . "KB";
-   }
+   $total_foldersize=lenstr($total_foldersize,0);
+
    $temphtml = qq|<tr>|.
                qq|<td align="center" bgcolor=$bgcolor><B>$lang_text{'total'}</B></td>|.
                qq|<td align="center" bgcolor=$bgcolor><B>$total_newmessages</B></td>|.
-               qq|<td align="center" bgcolor=$bgcolor><B>$total_allmessages</B></td>|.
-               qq|<td align="center" bgcolor=$bgcolor><B>$total_foldersize</B></td>|.
+               qq|<td align="center" bgcolor=$bgcolor><B>&nbsp;$total_allmessages</B></td>|.
+               qq|<td align="center" bgcolor=$bgcolor><B>&nbsp;$total_foldersize</B></td>|.
                qq|<td bgcolor=$bgcolor align="center">$usagestr</td>|.
                qq|</tr>|;
    $html =~ s/\@\@\@TOTAL\@\@\@/$temphtml/;
 
-   print $html;
-
-   printfooter();
+   print htmlheader(), $html, htmlfooter(1);
 }
 
 # this is inline function used by sub editfolders(), it changes
 # $total_newmessages, $total_allmessages and $total_size in editfolders()
 sub _folderline {
-   my ($currfolder, $i, $bgcolor)=@_;
+   my ($currfolder, $i, $bgcolor,
+       $r_total_newmessages, $r_total_allmessages, $r_total_foldersize)=@_;
    my $temphtml='';
    my (%HDB, $newmessages, $allmessages, $foldersize);
    my ($folderfile,$headerdb)=get_folderfile_headerdb($user, $currfolder);
 
-   if ( -f "$headerdb$config{'dbm_ext'}" ) {
-      filelock("$headerdb$config{'dbm_ext'}", LOCK_SH);
-      dbmopen (%HDB, $headerdb, undef);
+   if ( -f "$headerdb$config{'dbm_ext'}" && !-z "$headerdb$config{'dbm_ext'}" ) {
+      if (!$config{'dbmopen_haslock'}) {
+         filelock("$headerdb$config{'dbm_ext'}", LOCK_SH) or
+            openwebmailerror("$lang_err{'couldnt_locksh'} $headerdb$config{'dbm_ext'}");
+      }
+      dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", undef);
       if ( defined($HDB{'ALLMESSAGES'}) ) {
          $allmessages=$HDB{'ALLMESSAGES'};
-         $total_allmessages+=$allmessages;
+         ${$r_total_allmessages}+=$allmessages;
       } else {
          $allmessages='&nbsp;';
       }
       if ( defined($HDB{'NEWMESSAGES'}) ) {
          $newmessages=$HDB{'NEWMESSAGES'};
-         $total_newmessages+=$newmessages;
+         ${$r_total_newmessages}+=$newmessages;
       } else {
          $newmessages='&nbsp;';
       }
       dbmclose(%HDB);
-      filelock("$headerdb$config{'dbm_ext'}", LOCK_UN);
+      filelock("$headerdb$config{'dbm_ext'}", LOCK_UN) if (!$config{'dbmopen_haslock'});
    } else {
       $allmessages='&nbsp;';
       $newmessages='&nbsp;';
@@ -236,33 +232,37 @@ sub _folderline {
    # we count size for both folder file and related dbm
    $foldersize = (-s "$folderfile") + (-s "$headerdb$config{'dbm_ext'}");
 
-   $total_foldersize+=$foldersize;
-   # round foldersize and change to an appropriate unit for display
-   if ($foldersize > 1048575){
-      $foldersize = int(($foldersize/1048576)+0.5) . "MB";
-   } elsif ($foldersize > 1023) {
-      $foldersize =  int(($foldersize/1024)+0.5) . "KB";
-   }
+   ${$r_total_foldersize}+=$foldersize;
+   $foldersize=lenstr($foldersize,0);
 
    my $escapedcurrfolder = escapeURL($currfolder);
    my $url = "$config{'ow_cgiurl'}/openwebmail-folder.pl?sessionid=$thissession&amp;folder=$escapedcurrfolder&amp;action=downloadfolder";
    my $folderstr=$currfolder;
    $folderstr=$lang_folders{$currfolder} if defined($lang_folders{$currfolder});
 
+   my $accesskeystr=$i%10+1;
+   if ($accesskeystr == 10) {
+      $accesskeystr=qq|accesskey="0"|;
+   } elsif ($accesskeystr < 10) {
+      $accesskeystr=qq|accesskey="$accesskeystr"|;
+   }
+
    $temphtml .= qq|<tr>|.
                 qq|<td align="center" bgcolor=$bgcolor>|.
-                qq|<a href="$config{'ow_cgiurl'}/openwebmail-main.pl?action=displayheaders&amp;sessionid=$thissession&amp;sort=$sort&amp;firstmessage=$firstmessage&amp;folder=$escapedcurrfolder">|.
-                qq|$folderstr</a>&nbsp;\n|.
-                qq|<a href="$url" title="$lang_text{'download'} $folderstr"><IMG SRC="$config{'ow_htmlurl'}/images/iconsets/$prefs{'iconset'}/download.gif" align="absmiddle" border="0" ALT="$lang_text{'download'} $folderstr">|.
-                qq|</a></td>\n|.
+                qq|<a href="$config{'ow_cgiurl'}/openwebmail-main.pl?action=listmessages&amp;sessionid=$thissession&amp;sort=$sort&amp;page=$page&amp;folder=$escapedcurrfolder">|.
+                qq|$folderstr </a>&nbsp;\n|.
+                iconlink("download.gif", "$lang_text{'download'} $folderstr ", qq|$accesskeystr href="$url"|).
+                qq|</td>\n|.
                 qq|<td align="center" bgcolor=$bgcolor>$newmessages</td>|.
-                qq|<td align="center" bgcolor=$bgcolor>$allmessages</td>|.
-                qq|<td align="center" bgcolor=$bgcolor>$foldersize</td>\n|;
+                qq|<td align="center" bgcolor=$bgcolor>&nbsp;$allmessages</td>|.
+                qq|<td align="center" bgcolor=$bgcolor>&nbsp;$foldersize</td>\n|;
+
+   $temphtml .= qq|<td bgcolor=$bgcolor align="center">\n|;
 
    $temphtml .= start_form(-action=>"$config{'ow_cgiurl'}/openwebmail-folder.pl",
                            -name=>"folderform$i");
    $temphtml .= hidden(-name=>'action',
-                       -value=>'deletefolder',
+                       -value=>'chkindexfolder',
                        -override=>'1');
    $temphtml .= hidden(-name=>'sessionid',
                        -value=>$thissession,
@@ -270,8 +270,8 @@ sub _folderline {
    $temphtml .= hidden(-name=>'sort',
                        -default=>$sort,
                        -override=>'1');
-   $temphtml .= hidden(-name=>'firstmessage',
-                       -default=>$firstmessage,
+   $temphtml .= hidden(-name=>'page',
+                       -default=>$page,
                        -override=>'1');
    $temphtml .= hidden(-name=>'folder',
                        -default=>$folder,
@@ -282,38 +282,146 @@ sub _folderline {
    $temphtml .= hidden(-name=>'foldernewname',
                        -value=>$currfolder,
                        -override=>'1');
+   $temphtml .= "\n";
 
-   $temphtml .= qq|\n<td bgcolor=$bgcolor align="center">|;
-
+   my $jsfolderstr=$lang_folders{$currfolder}||$currfolder;
+   $jsfolderstr=~ s/'/\\'/g;	# escaep ' with \'
+   $temphtml .= submit(-name=>"$lang_text{'markread'}",
+                       -class=>"medtext",
+                       -onClick=>"return OpConfirm('folderform$i', 'markreadfolder', $lang_text{'foldermarkreadconf'}+' ( $jsfolderstr )')");
    $temphtml .= submit(-name=>"$lang_text{'chkindex'}",
                        -class=>"medtext",
-                       -onClick=>"return OpConfirm('folderform$i', 'chkindexfolder', $lang_text{'folderchkindexconf'}+' ( $currfolder )')")."\n";
-   $temphtml .= submit(-name=>"$lang_text{'reindex'}", 
+                       -onClick=>"return OpConfirm('folderform$i', 'chkindexfolder', $lang_text{'folderchkindexconf'}+' ( $jsfolderstr )')");
+   $temphtml .= submit(-name=>"$lang_text{'reindex'}",
                        -class=>"medtext",
-                       -onClick=>"return OpConfirm('folderform$i', 'reindexfolder', $lang_text{'folderreindexconf'}+' ( $currfolder )')")."\n";
-
+                       -onClick=>"return OpConfirm('folderform$i', 'reindexfolder', $lang_text{'folderreindexconf'}+' ( $jsfolderstr )')");
    if ($currfolder ne "INBOX") {
-      $temphtml .= submit(-name=>"$lang_text{'rename'}", 
+      $temphtml .= submit(-name=>"$lang_text{'rename'}",
                           -class=>"medtext",
-                          -onClick=>"return OpConfirm('folderform$i', 'renamefolder', $lang_text{'folderrenprop'}+' ( $currfolder )')")."\n";
+                          -onClick=>"return OpConfirm('folderform$i', 'renamefolder', $lang_text{'folderrenprop'}+' ( $jsfolderstr )')");
       $temphtml .= submit(-name=>"$lang_text{'delete'}",
                           -class=>"medtext",
-                          -onClick=>"return OpConfirm('folderform$i', 'deletefolder', $lang_text{'folderdelconf'}+' ( $currfolder )')")."\n";
+                          -onClick=>"return OpConfirm('folderform$i', 'deletefolder', $lang_text{'folderdelconf'}+' ( $jsfolderstr )')");
    }
 
-   $temphtml .= '</td></tr>';
-   $temphtml .= end_form(). "\n";
+   $temphtml .= "</td></tr>";
+   $temphtml .= end_form()."\n";
 
    return($temphtml);
 }
 ################### END EDITFOLDERS ########################
 
+################### REFRESHFOLDERS ##############################
+sub refreshfolders {
+   my $errcount=0;
+
+   foreach my $currfolder (@validfolders) {
+      my ($folderfile,$headerdb)=get_folderfile_headerdb($user, $currfolder);
+
+      filelock($folderfile, LOCK_EX|LOCK_NB) or
+         openwebmailerror("$lang_err{'couldnt_lock'} $folderfile!");
+      if (update_headerdb($headerdb, $folderfile)<0) {
+         $errcount++;
+         writelog("db error - Couldn't update index db $headerdb$config{'dbm_ext'}");
+         writehistory("db error - Couldn't update index db $headerdb$config{'dbm_ext'}");
+      }
+      filelock($folderfile, LOCK_UN);
+   }
+
+   writelog("refresh folders - $errcount errors");
+   writehistory("refresh folders - $errcount errors");
+
+   getfolders(\@validfolders, \$folderusage);
+   editfolders();
+}
+################### END REFRESHFOLDERS ##########################
+
+################### MARKREADFOLDER ##############################
+sub markreadfolder {
+   my $foldertomark = safefoldername(param('foldername')) || '';
+   ($foldertomark =~ /^(.+)$/) && ($foldertomark = $1);
+   my ($folderfile, $headerdb)=get_folderfile_headerdb($user, $foldertomark);
+
+   filelock($folderfile, LOCK_EX|LOCK_NB) or
+      openwebmailerror("$lang_err{'couldnt_lock'} $folderfile!");
+
+   if (update_headerdb($headerdb, $folderfile)<0) {
+      filelock($folderfile, LOCK_UN);
+      openwebmailerror("$lang_err{'couldnt_updatedb'} $headerdb$config{'dbm_ext'}");
+   }
+
+   my (%HDB, %offset, %status);
+   if (!$config{'dbmopen_haslock'}) {
+      filelock("$headerdb$config{'dbm_ext'}", LOCK_SH) or
+         openwebmailerror("$lang_err{'couldnt_locksh'} $headerdb$config{'dbm_ext'}");
+   }
+   dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", undef);
+   foreach my $messageid (keys %HDB) {
+      next if ( $messageid eq 'METAINFO'
+             || $messageid eq 'NEWMESSAGES'
+             || $messageid eq 'INTERNALMESSAGES'
+             || $messageid eq 'ALLMESSAGES'
+             || $messageid eq "" );
+      my @attr=split( /@@@/, $HDB{$messageid} );
+      if ($attr[$_STATUS] !~ /R/i) {
+         $offset{$messageid}=$attr[$_OFFSET];
+         $status{$messageid}=$attr[$_STATUS];
+      }
+   }
+   dbmclose(%HDB);
+   filelock("$headerdb$config{'dbm_ext'}", LOCK_UN) if (!$config{'dbmopen_haslock'});
+
+   my @markids;
+   my $tmpfile="/tmp/markread_tmp_$$";
+   my $tmpdb="/tmp/.markread_tmp_$$";
+   ($tmpfile =~ /^(.+)$/) && ($tmpfile = $1);
+   ($tmpdb =~ /^(.+)$/) && ($tmpdb = $1);
+
+   filelock("$tmpfile", LOCK_EX) or
+      openwebmailerror("$lang_err{'couldnt_lock'} $tmpfile");
+
+   if (update_headerdb($tmpdb, $tmpfile)<0) {
+      filelock($tmpfile, LOCK_UN);
+      openwebmailerror("$lang_err{'couldnt_updatedb'} $tmpdb$config{'dbm_ext'}");
+   }
+
+   foreach my $messageid (sort { $offset{$a}<=>$offset{$b} } keys %offset) {
+      my @copyid;
+      push(@copyid, $messageid);
+      if (operate_message_with_ids("copy", \@copyid,
+   					$folderfile, $headerdb, $tmpfile, $tmpdb) >0 ) {
+         update_message_status($messageid, $status{$messageid}."R", $tmpdb, $tmpfile);
+         push(@markids, $messageid);
+
+         if ($#markids>=99) { # flush per 100 msgs
+            operate_message_with_ids("delete", \@markids, $folderfile, $headerdb);
+            operate_message_with_ids("move", \@markids,
+   					$tmpfile, $tmpdb, $folderfile, $headerdb);
+            @markids=();
+         }
+
+      }
+   }
+   operate_message_with_ids("delete", \@markids, $folderfile, $headerdb);
+   operate_message_with_ids("move", \@markids,
+   					$tmpfile, $tmpdb, $folderfile, $headerdb);
+
+   filelock("$tmpfile", LOCK_UN);
+   filelock($folderfile, LOCK_UN);
+   unlink("$tmpdb$config{'dbm_ext'}", $tmpfile);
+
+   writelog("markread folder - $foldertomark");
+   writehistory("markread folder - $foldertomark");
+
+   getfolders(\@validfolders, \$folderusage);
+   editfolders();
+}
+################### END MARKREADFOLDER ##########################
+
 ################### REINDEXFOLDER ##############################
 sub reindexfolder {
    my $recreate=$_[0];
-   my $foldertoindex = param('foldername') || '';
-   $foldertoindex =~ s/\.\.+//g;
-   $foldertoindex =~ s/[\s\/\`\|\<\>;]//g; # remove dangerous char
+   my $foldertoindex = safefoldername(param('foldername')) || '';
    ($foldertoindex =~ /^(.+)$/) && ($foldertoindex = $1);
    my ($folderfile, $headerdb)=get_folderfile_headerdb($user, $foldertoindex);
 
@@ -325,26 +433,37 @@ sub reindexfolder {
 
    if ( -f "$headerdb$config{'dbm_ext'}" ) {
       my %HDB;
-      filelock("$headerdb$config{'dbm_ext'}", LOCK_SH);
-      dbmopen (%HDB, $headerdb, undef);
+      if (!$config{'dbmopen_haslock'}) {
+         filelock("$headerdb$config{'dbm_ext'}", LOCK_SH) or
+            openwebmailerror("$lang_err{'couldnt_locksh'} $headerdb$config{'dbm_ext'}");
+      }
+      dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", 0600);
       $HDB{'METAINFO'}={'RENEW'};
       dbmclose(%HDB);
-      filelock("$headerdb$config{'dbm_ext'}", LOCK_UN);
+      filelock("$headerdb$config{'dbm_ext'}", LOCK_UN) if (!$config{'dbmopen_haslock'});
+   }
+   if (update_headerdb($headerdb, $folderfile)<0) {
+      filelock($folderfile, LOCK_UN);
+      openwebmailerror("$lang_err{'couldnt_updatedb'} $headerdb$config{'dbm_ext'}");
    }
-   update_headerdb($headerdb, $folderfile);
 
+   if ($recreate) {
+      writelog("reindex folder - $foldertoindex");
+      writehistory("reindex folder - $foldertoindex");
+   } else {
+      writelog("chkindex folder - $foldertoindex");
+      writehistory("chkindex folder - $foldertoindex");
+   }
+
+#   print "Location: $config{'ow_cgiurl'}/openwebmail-prefs.pl?action=editfolders&sessionid=$thissession&sort=$sort&folder=$escapedfolder&page=$page\n\n";
    getfolders(\@validfolders, \$folderusage);
    editfolders();
-
-#   print "Location: $config{'ow_cgiurl'}/openwebmail-prefs.pl?action=editfolders&sessionid=$thissession&sort=$sort&folder=$escapedfolder&firstmessage=$firstmessage\n\n";
 }
 ################### END REINDEXFOLDER ##########################
 
 ################### ADDFOLDER ##############################
 sub addfolder {
-   my $foldertoadd = param('foldername') || '';
-   $foldertoadd =~ s/\.\.+//g;
-   $foldertoadd =~ s/[\s\/\`\|\<\>;]//g; # remove dangerous char
+   my $foldertoadd = safefoldername(param('foldername')) || '';
    ($foldertoadd =~ /^(.+)$/) && ($foldertoadd = $1);
 
    if (length($foldertoadd) > $config{'foldername_maxlen'}) {
@@ -366,7 +485,7 @@ sub addfolder {
 
    # create empty index dbm with mode 0600
    my %HDB;
-   dbmopen(%HDB, $headerdb, 0600);
+   dbmopen(%HDB, "$headerdb$config{'dbmopen_ext'}", 0600);
    dbmclose(%HDB);
 
    writelog("create folder - $foldertoadd");
@@ -379,7 +498,7 @@ sub addfolder {
 sub is_defaultfolder {
    my $foldername=$_[0];
    if ($foldername eq 'INBOX' ||
-       $foldername eq 'saved-messages' || 
+       $foldername eq 'saved-messages' ||
        $foldername eq 'sent-mail' ||
        $foldername eq 'saved-drafts' ||
        $foldername eq 'mail-trash' ||
@@ -388,18 +507,16 @@ sub is_defaultfolder {
        $foldername eq $lang_folders{'sent-mail'} ||
        $foldername eq $lang_folders{'saved-drafts'} ||
        $foldername eq $lang_folders{'mail-trash'} ) {
-      return(1);
+      return 1;
    } else {
-      return(0);
+      return 0;
    }
 }
 ################### END ADDFOLDER ##########################
 
 ################### DELETEFOLDER ##############################
 sub deletefolder {
-   my $foldertodel = param('foldername') || '';
-   $foldertodel =~ s/\.\.+//g;
-   $foldertodel =~ s/[\s\/\`\|\<\>;]//g; # remove dangerous char
+   my $foldertodel = safefoldername(param('foldername')) || '';
    ($foldertodel =~ /^(.+)$/) && ($foldertodel = $1);
 
    # if is INBOX, return to editfolder immediately
@@ -409,14 +526,16 @@ sub deletefolder {
    }
 
    if ( -f "$folderdir/$foldertodel" ) {
+      my $headerdb="$folderdir/$foldertodel";
+      ($headerdb =~ /^(.+)\/(.*)$/) && ($headerdb = "$1/.$2");
       unlink ("$folderdir/$foldertodel",
-              "$folderdir/.$foldertodel$config{'dbm_ext'}",
-              "$folderdir/.$foldertodel.db",
-	      "$folderdir/.$foldertodel.dir",
-              "$folderdir/.$foldertodel.pag",
-              "$folderdir/.$foldertodel.cache",
               "$folderdir/$foldertodel.lock",
-              "$folderdir/$foldertodel.lock.lock");              
+              "$folderdir/$foldertodel.lock.lock",
+              "$headerdb$config{'dbm_ext'}",
+              "$headerdb.db",
+	      "$headerdb.dir",
+              "$headerdb.pag",
+              "$headerdb.cache");
 
       writelog("delete folder - $foldertodel");
       writehistory("delete folder - $foldertodel");
@@ -429,19 +548,14 @@ sub deletefolder {
 
 ################### RENAMEFOLDER ##############################
 sub renamefolder {
-   my $oldname = param('foldername') || '';
-   $oldname =~ s/\.\.+//g;
-   $oldname =~ s/[\s\/\`\|\<\>;]//g; # remove dangerous char
+   my $oldname = safefoldername(param('foldername')) || '';
    ($oldname =~ /^(.+)$/) && ($oldname = $1);
-
    if ($oldname eq 'INBOX') {
       editfolders();
       return;
    }
 
-   my $newname = param('foldernewname');
-   $newname =~ s/\.\.+//g;
-   $newname =~ s/[\s\/\`\|\<\>;]//g; # remove dangerous char
+   my $newname = safefoldername(param('foldernewname'));
    ($newname =~ /^(.+)$/) && ($newname = $1);
 
    if (length($newname) > $config{'foldername_maxlen'}) {
@@ -456,19 +570,23 @@ sub renamefolder {
    }
 
    if ( -f "$folderdir/$oldname" ) {
-      rename("$folderdir/$oldname",          "$folderdir/$newname");
-      rename("$folderdir/.$oldname$config{'dbm_ext'}", "$folderdir/.$newname$config{'dbm_ext'}");
-      rename("$folderdir/.$oldname.db",      "$folderdir/.$newname.db");
-      rename("$folderdir/.$oldname.dir",     "$folderdir/.$newname.dir");
-      rename("$folderdir/.$oldname.pag",     "$folderdir/.$newname.pag");
-      rename("$folderdir/.$oldname.cache",   "$folderdir/.$newname.cache");
-      unlink("$folderdir/$oldname.lock", "$folderdir/$oldname.lock.lock");
+      my $oldheaderdb="$folderdir/$oldname";
+      my $newheaderdb="$folderdir/$newname";
+      ($oldheaderdb =~ /^(.+)\/(.*)$/) && ($oldheaderdb = "$1/.$2");
+      ($newheaderdb =~ /^(.+)\/(.*)$/) && ($newheaderdb = "$1/.$2");
+      rename("$folderdir/$oldname",            "$folderdir/$newname");
+      rename("$oldheaderdb$config{'dbm_ext'}", "$newheaderdb$config{'dbm_ext'}");
+      rename("$oldheaderdb.db",                "$newheaderdb.db");
+      rename("$oldheaderdb.dir",               "$newheaderdb.dir");
+      rename("$oldheaderdb.pag",               "$newheaderdb.pag");
+      rename("$oldheaderdb.cache",             "$newheaderdb.cache");
+      unlink("$folderdir/$newname.lock",       "$folderdir/$oldname.lock.lock");
 
       writelog("rename folder - rename $oldname to $newname");
       writehistory("rename folder - rename $oldname to $newname");
    }
 
-#   print "Location: $config{'ow_cgiurl'}/openwebmail-prefs.pl?action=editfolders&sessionid=$thissession&sort=$sort&folder=$escapedfolder&firstmessage=$firstmessage\n\n";
+#   print "Location: $config{'ow_cgiurl'}/openwebmail-prefs.pl?action=editfolders&sessionid=$thissession&sort=$sort&folder=$escapedfolder&page=$page\n\n";
    getfolders(\@validfolders, \$folderusage);
    editfolders();
 }
@@ -481,12 +599,12 @@ sub downloadfolder {
    my $buff;
 
    if ( -x '/usr/local/bin/zip' ) {
-      $cmd="/usr/local/bin/zip -r - $folderfile |";
+      $cmd="/usr/local/bin/zip -rq - $folderfile |";
       $contenttype='application/x-zip-compressed';
       $filename="$folder.zip";
 
    } elsif ( -x '/usr/bin/zip' ) {
-      $cmd="/usr/bin/zip -r - $folderfile |";
+      $cmd="/usr/bin/zip -rq - $folderfile |";
       $contenttype='application/x-zip-compressed';
       $filename="$folder.zip";
 
@@ -515,14 +633,12 @@ sub downloadfolder {
    print qq|Content-Transfer-Coding: binary\n|,
          qq|Connection: close\n|,
          qq|Content-Type: $contenttype; name="$filename"\n|;
-
-   # ugly hack since ie5.5 is broken with disposition: attchment
-   if ( $ENV{'HTTP_USER_AGENT'}!~/MSIE 5.5/ ) {
+   if ( $ENV{'HTTP_USER_AGENT'}!~/MSIE 5.5/ ) {	# ie5.5 is broken with content-disposition
       print qq|Content-Disposition: attachment; filename="$filename"\n|;
    }
    print qq|\n|;
 
-   ($cmd =~ /^(.+)$/) && ($cmd = $1);		# bypass taint check
+   ($cmd =~ /^(.+)$/) && ($cmd = $1);		# untaint ...
    open (T, $cmd);
    while ( read(T, $buff,32768) ) {
      print $buff;
