@@ -1,5 +1,5 @@
 #
-# pop3mail.pl - functions for pop3 mail retrieval
+# pop3mail.pl - pop3 mail retrieval routines
 #
 # 2002/03/19 eddie@turtle.ee.ncku.edu.tw
 #            tung@turtle.ee.ncku.edu.tw
@@ -7,8 +7,8 @@
 
 use strict;
 use Fcntl qw(:DEFAULT :flock);
-use FileHandle;
 use IO::Socket;
+require "mime.pl";
 
 use vars qw(%config);
 
@@ -32,16 +32,16 @@ sub retrpop3mail {
    my ($dummy, $i);
 
    if ( readpop3book($pop3book, \%accounts)<0 ) {
-      return(-1);
+      return -1;
    }
 
    ($dummy, $dummy, $pop3passwd, $pop3lastid, $pop3del, $enable)=
-			split(/\@\@\@/, $accounts{"$pop3host:$pop3user"});
+			split(/\@\@\@/, $accounts{"$pop3host\@\@\@$pop3user"});
 
    # untaint for file creation
-   ($spoolfile =~ /^(.+)$/) && ($spoolfile = $1); 	
+   ($spoolfile =~ /^(.+)$/) && ($spoolfile = $1);
    # untaint for connection creation
-   ($pop3host =~ /^(.+)$/) && ($pop3host = $1); 	
+   ($pop3host =~ /^(.+)$/) && ($pop3host = $1);
 
    $ServerPort='110';
    eval {
@@ -52,28 +52,39 @@ sub retrpop3mail {
                                            PeerPort=>$ServerPort,);
       alarm 0;
    };
-   return(-2) if ($@);			# eval error, it means timeout
-   return(-2) if (!$remote_sock);	# connect error
+   return -2 if ($@);			# eval error, it means timeout
+   return -2 if (!$remote_sock);	# connect error
 
    $remote_sock->autoflush(1);
    $_=<$remote_sock>;
-   return(-3) if (/^\-/);		# server not ready
+   return -3 if (/^\-/);		# server not ready
 
-   print $remote_sock "user $pop3user\r\n";
+   # try if server supports auth login(base64 encoding) first
+   print $remote_sock "auth login\r\n";
    $_=<$remote_sock>;
-   return(-4) if (/^\-/);		# username error
-
-   print $remote_sock "pass $pop3passwd\r\n";
-   $_=<$remote_sock>;
-   return (-5) if (/^\-/);		# passwd error
-
+   if (/^\+/) {
+      print $remote_sock &encode_base64($pop3user);
+      $_=<$remote_sock>;
+      (close($remote_sock) && return -4) if (/^\-/);		# username error
+      print $remote_sock &encode_base64($pop3passwd);
+      $_=<$remote_sock>;
+   }
+   if (! /^\+/) {	# not supporting auth login or auth login failed
+      print $remote_sock "user $pop3user\r\n";
+      $_=<$remote_sock>;
+      (close($remote_sock) && return -4) if (/^\-/);		# username error
+      print $remote_sock "pass $pop3passwd\r\n";
+      $_=<$remote_sock>;
+      (close($remote_sock) && return -5) if (/^\-/);		# passwd error
+   }
    print $remote_sock "stat\r\n";
    $_=<$remote_sock>;
-   return(-6) if (/^\-/);		# stat error
+   (close($remote_sock) && return -6) if (/^\-/);		# stat error
 
    $nMailCount=(split(/\s/))[1];
    if ($nMailCount == 0) {		# no message
       print $remote_sock "quit\r\n";
+      close($remote_sock);
       return 0;
    }
 
@@ -93,6 +104,7 @@ sub retrpop3mail {
          $last=(split(/\s/))[1];		# +OK N
          if ($last eq $nMailCount) {
             print $remote_sock "quit\r\n";
+            close($remote_sock);
             return 0;
          }
       }
@@ -107,6 +119,7 @@ sub retrpop3mail {
 
       if ($pop3lastid eq (split(/\s/))[$uidl_field]) {	# +OK N ID
          print $remote_sock "quit\r\n";
+         close($remote_sock);
          return 0;
       }
       if ($pop3lastid ne "none") {
@@ -139,7 +152,8 @@ sub retrpop3mail {
          if ( /^\+/ ) {
             next;
          } elsif (/^\-/) {
-            return(-7);
+            close($remote_sock);
+            return -7;
          } else {
             last;
          }
@@ -168,7 +182,7 @@ sub retrpop3mail {
                $_ = $2;
             } elsif ($_=~ /<?(.*@.*)>?\s+\((.+?)\)/ ) {
                $_ = $1;
-            } elsif ($_=~ /<\s*(.+@.+)\s*>/ ) { 
+            } elsif ($_=~ /<\s*(.+@.+)\s*>/ ) {
                $_ = $1;
             } else {
                $_=~ s/\s*(.+@.+)\s*/$1/;
@@ -194,11 +208,11 @@ sub retrpop3mail {
       }
 
       # append message to mail folder
-      filelock($spoolfile, LOCK_EX);
-      open(IN,">>$spoolfile") or return(-8);
+      filelock($spoolfile, LOCK_EX) or return -8;
+      open(IN,">>$spoolfile") or return -8;
       print IN "From $stAddress $stDate\n";
       print IN $FileContent;
-      print IN "\n";		# mark mail end 
+      print IN "\n";		# mark mail end
       close(IN);
       filelock($spoolfile, LOCK_UN);
 
@@ -221,13 +235,13 @@ sub retrpop3mail {
    close($remote_sock);
 
    ###  write back to pop3book
-   $accounts{"$pop3host:$pop3user"} = "$pop3host\@\@\@$pop3user\@\@\@$pop3passwd\@\@\@$pop3lastid\@\@\@$pop3del\@\@\@$enable";
+   $accounts{"$pop3host\@\@\@$pop3user"} = "$pop3host\@\@\@$pop3user\@\@\@$pop3passwd\@\@\@$pop3lastid\@\@\@$pop3del\@\@\@$enable";
    if (writepop3book($pop3book, \%accounts)<0) {
-      return(-9);
+      return -9;
    }
 
    # return number of fetched mail
-   return($retr_total);		
+   return($retr_total);
 }
 
 1;
